<?php

    /***************************************************************************
    *                            Dolphin Smart Community Builder
    *                              -------------------
    *     begin                : Mon Mar 23 2006
    *     copyright            : (C) 2007 BoonEx Group
    *     website              : http://www.boonex.com
    * This file is part of Dolphin - Smart Community Builder
    *
    * Dolphin is free software; you can redistribute it and/or modify it under
    * the terms of the GNU General Public License as published by the
    * Free Software Foundation; either version 2 of the
    * License, or  any later version.
    *
    * Dolphin is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
    * without even the implied warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    * See the GNU General Public License for more details.
    * You should have received a copy of the GNU General Public License along with Dolphin,
    * see license.txt file; if not, write to marketing@boonex.com
    ***************************************************************************/

    require_once( BX_DIRECTORY_PATH_INC . 'profiles.inc.php' );

    bx_import('BxDolModuleDb');
    bx_import('BxDolModule');
    bx_import('BxDolInstallerUtils');

    class BxFaceBookConnectModule extends BxDolModule 
    {
        // contain some module information ;
        var $aModuleInfo;

        // contain path for current module;
        var $sPathToModule;
        var $sHomeUrl;

        var $oFacebook;

        // array contain alternative profiles name 
        //(need if dolphin already contain profile name such as facebook's profile name)
        var $aAlternativeNames;

        /**
    	 * Class constructor ;
         *
         * @param   : $aModule (array) - contain some information about this module;
         *                  [ id ]           - (integer) module's  id ;
         *                  [ title ]        - (string)  module's  title ;
         *                  [ vendor ]       - (string)  module's  vendor ;
         *                  [ path ]         - (string)  path to this module ;
         *                  [ uri ]          - (string)  this module's URI ;
         *                  [ class_prefix ] - (string)  this module's php classes file prefix ;
         *                  [ db_prefix ]    - (string)  this module's Db tables prefix ;
         *                  [ date ]         - (string)  this module's date installation ;
    	 */
    	function BxFaceBookConnectModule(&$aModule) 
        {
            parent::BxDolModule($aModule);

            require_once(BX_DIRECTORY_PATH_MODULES . $aModule['path'] . '/inc/facebook.php');

            // prepare the location link ;
            $this -> sPathToModule  = BX_DOL_URL_ROOT . $this -> _oConfig -> getBaseUri();
            $this -> aModuleInfo    = $aModule;

            $this -> sHomeUrl       = $this ->_oConfig -> _sHomeUrl;

            // Create our Application instance.
			$this -> oFacebook  = new Facebook(array(
				  'appId'  => $this -> _oConfig -> mApiID,
				  'secret' => $this -> _oConfig -> mApiSecret,
				  'cookie' => true,
			));

            $this -> aAlternativeNames = array(
                '_fb_id', '_facebook'
            );
        }

        /**
         * Function will generate facebook's admin page;
         *
         * @return : (text) - html presentation data; 
         */
        function actionAdministration()
        {
        	$GLOBALS['iAdminPage'] = 1;

            if( !isAdmin() ) {
                header('location: ' . BX_DOL_URL_ROOT);
            }

            // get sys_option's category id;
            $iCatId = $this-> _oDb -> getSettingsCategoryId('bx_facebook_connect_api_key');
            if(!$iCatId) {
                $sOptions = MsgBox( _t('_Empty') );
            }
            else {
                bx_import('BxDolAdminSettings');

                $oSettings = new BxDolAdminSettings($iCatId);
                
                $mixedResult = '';
                if(isset($_POST['save']) && isset($_POST['cat'])) {
                    $mixedResult = $oSettings -> saveChanges($_POST);
                }

                // get option's form;
                $sOptions = $oSettings -> getForm();
                if($mixedResult !== true && !empty($mixedResult)) {
                    $sOptions = $mixedResult . $sOptions;
                }
            }

            $sCssStyles = $this -> _oTemplate -> addCss('forms_adv.css', true);

            $this -> _oTemplate-> pageCodeAdminStart();

                echo DesignBoxAdmin( _t('_bx_facebook_information')
                        , $GLOBALS['oSysTemplate'] -> parseHtmlByName('default_padding.html', array('content' => _t('_bx_facebook_information_block'))) );
                echo DesignBoxAdmin( _t('_Settings')
                        , $GLOBALS['oSysTemplate'] -> parseHtmlByName('default_padding.html', array('content' => $sCssStyles . $sOptions) ));

            $this -> _oTemplate->pageCodeAdmin( _t('_bx_facebook_settings') );
        }

        /**
         * Function will get the facebook login form;
         *
         * @return : (text) - html presentation data;
         */
        function actionLoginForm()
        {
        	if( getLoggedId() ) {
        		header('location:'. BX_DOL_URL_ROOT);
        	}

            // check Fb api keys;
            if(!$this -> _oConfig -> mApiID || !$this -> _oConfig -> mApiSecret) {
                echo $this -> _oTemplate -> getPage( _t('_bx_facebook_error_occured')
                          , MsgBox( _t('_bx_facebook_profile_error_api_keys') ));
                exit;
            }

            $sSession = $this -> oFacebook -> getSession();

			if(!$sSession) {
				//step one
				header('location: ' . $this -> oFacebook -> getLoginUrl());
			}
			else {
				try {
					$aFacebookProfileInfo = $this -> oFacebook -> api('/me?fields=picture');
				} catch (FacebookApiException $e) {
					  // profile not defined;
	                echo $this -> _oTemplate -> getPage( _t('_bx_facebook_error_occured')
	                        , MsgBox($e));
	                exit;  
				}

				//procces profile info
				if($aFacebookProfileInfo) {
					 // get dolphin's profile Id;
                    // @see : $this -> aFacebookProfileFields;
                    $aFacebookProfileInfo['first_name'] 
                    	= $this -> _proccesNickName($aFacebookProfileInfo['first_name']);

					$aFacebookProfileInfo['generate_uid'] = md5($aFacebookProfileInfo['proxied_email'] . $aFacebookProfileInfo['first_name']);

				    // try define user id 
                    $iProfileId = $this -> _oDb -> getProfileId($aFacebookProfileInfo['generate_uid']);
                    if($iProfileId) {
                   		// get profile info
                   		$aDolphinProfileInfo = getProfileInfo($iProfileId);
                   		$this -> setLogged($iProfileId, $aDolphinProfileInfo['Password']);
                    }
                    else {
                        //profile not defined
                   		if( !getID($aFacebookProfileInfo['first_name']) ) {
                   			// create new ;
                   			$iProfileId = $this -> _createProfile($aFacebookProfileInfo);
                   		}
                   		else {
                   			//-- profile nickname already used by other person --//

                   			$sAlternativeName = $this -> getAlternativeName($aFacebookProfileInfo['first_name']);
                            if($sAlternativeName) {
                                // create new profile;
                                $iProfileId = $this -> _createProfile($aFacebookProfileInfo, $sAlternativeName);
                            }
                            else {
                                // generate error : profile exist;
                                echo $this -> _oTemplate -> getPage( _t('_bx_facebook_error_occured')
                                        , MsgBox( _t('_bx_facebook_profile_exist', $aFacebookProfileInfo['first_name']) ));
                                exit;
                            }
							//--
                   		}
                    }
				}
				else {
					// profile info not defined;
                    echo $this -> _oTemplate -> getPage( _t('_bx_facebook_error_occured')
                            , MsgBox( _t('_bx_facebook_profile_error_info') ));
                    exit;   
				}
			}
        }

        /**
         * Function will set profile as logged;
         *
         * @param  : $iProfileId (integer) - profile's Id;
         * @param  : $sPassword  (string)  - profile's password;
         * @param  : $sCallbackUrl  (string) - redirect url that will use after profile logged;
         * @return : void;
         */
        function setLogged($iProfileId, $sPassword, $sCallbackUrl = null)
        {
            bx_login($iProfileId);
            $sCallbackUrl = ($sCallbackUrl) ? $sCallbackUrl : BX_DOL_URL_ROOT;
            header('location: ' . $sCallbackUrl);
        }

        /**
         * Function will get profile's alternative name
         * for join in dolphin DB;
         *
         * @param : $sName (string) - current profile's name;
         */
        function getAlternativeName($sName)
        {
            $sRetName = null;

            foreach($this -> aAlternativeNames as $sAlternativeName)
            {
                $iProfileId = getID($sName . $sAlternativeName);
                if(!$iProfileId) {
                    $sRetName =  $sAlternativeName;
                    break;
                }
            }

            return $sRetName;
        }

        /**
         * Function will create new profile;
         *
         * @param  : $aProfileInfo (array) - some profile's information; 
         *          @see : $this -> aFacebookProfileFields;
         * @param  : $sAlternativeName (string) - profiles alternative name; 
         * @return : (integer) profile's id; 
         */
        function _createProfile(&$aProfileInfo, $sAlternativeName = null)
        {
            $aClearedInfo = array();
			$sCity = $sCountry = '';

            if(!$aProfileInfo || !$aProfileInfo['first_name']) {
                return;
            }

            // procces the date of birth;
            if( isset($aProfileInfo['birthday']) ) {
                $aProfileInfo['birthday'] = isset($aProfileInfo['birthday'])
                	?  date('Y-m-d', strtotime($aProfileInfo['birthday']) )
                	:  '';
            }

            // generate new password for profile;
            $sSalt = genRndSalt();
          	$aProfileInfo['password'] = encryptUserPwd(genRndPwd(), $sSalt);

          	// define nick name;
          	$sNickName = $aProfileInfo['first_name'] . $sAlternativeName;

          	//define user's country and city
          	if( isset($aProfileInfo['hometown']['name']) ) {
          		$aCountryInfo = explode(',', $aProfileInfo['hometown']['name']);
          		$sCountry = $this -> _oDb -> getCountryCode( trim($aCountryInfo[1]) );
          		$sCity = trim($aCountryInfo[0]); 
          	}

            // fill array with all needed values;
            $aProfileFields = array(
                'NickName'      		=> $sNickName,
                'Email'         		=> isset($aProfileInfo['email']) ? $aProfileInfo['email'] : '',
                'Sex'           		=> isset($aProfileInfo['gender']) ? $aProfileInfo['gender'] : '',
                'DateOfBirth'   		=> $aProfileInfo['birthday'],

                'Password'      		=> $aProfileInfo['password'],

            	'FirstName'				=> isset($aProfileInfo['first_name']) ? $aProfileInfo['first_name'] : '',
                'LastName'				=> isset($aProfileInfo['last_name']) ? $aProfileInfo['last_name'] : '',

            	'DescriptionMe' 		=> isset($aProfileInfo['about']) ? $aProfileInfo['about'] : '',
                'Interests'     		=> isset($aProfileInfo['interests']) ? $aProfileInfo['interests'] : '',

                'Religion'      		=> isset($aProfileInfo['religion']) ? $aProfileInfo['religion'] : '',
                'Country'       		=> $sCountry,
            	'City'       			=> $sCity,
            );

            // check fields existence;
            foreach($aProfileFields as $sKey => $mValue) {
                if( !$this -> _oDb -> isFieldExist($sKey)) {
                    // (field not existence) remove from array;
                    unset($aProfileFields[$sKey]);
                }
            }

            // add some system values;
            $aProfileFields['Role'] = 1;

            $aProfileFields['FacebookProfile'] 		= $aProfileInfo['generate_uid'];
            $aProfileFields['DateReg'] 				= date( 'Y-m-d H:i:s' ); // set current date;

            // create new profile;
            $iProfileId = $this -> _oDb -> createProfile($aProfileFields);

            // check profile status;
            if ( getParam('autoApproval_ifNoConfEmail') == 'on' ) {
                if ( getParam('autoApproval_ifJoin') == 'on' ) {
                    $sProfileStatus = 'Active';
                }    
                else {
                    $sProfileStatus = 'Approval';
                }
            } 
            else {
                $sProfileStatus = 'Unconfirmed';
            }

            // update profile's status;
            $this -> _oDb -> updateProfileStatus($iProfileId, $sProfileStatus);

            // create system event
            bx_import('BxDolAlerts');
            $oZ = new BxDolAlerts('profile', 'join', $iProfileId);
            $oZ -> alert();

            // check avatar module;
            if( BxDolInstallerUtils::isModuleInstalled('avatar') ) {
                // check profile's logo;
                if( isset($aProfileInfo['picture']) && $aProfileInfo['picture'] ) {
                    BxDolService::call('avatar', 'set_image_for_cropping', array ($iProfileId, $aProfileInfo['picture'])); 
                }

                if (BxDolService::call('avatar', 'join', array ($iProfileId, '_Join complete'))) {
                    exit;
                }
            }
            else {
                // set logged and redirect on home page;
                $aProfileInfo = getProfileInfo($iProfileId);
                $this -> setLogged($iProfileId, $aProfileInfo['Password']);
            }
        }

        /**
         * Function will clear all unnecessary sybmols from profile's nickname;
         *
         * @param  : $sProfileName (string) - profile's nickname;
         * @return : (string) - cleared nickname;
         */
        function _proccesNickName($sProfileName)
        {
            $sProfileName = preg_replace("/^http:\/\/|^https:\/\/|\/$/", '', $sProfileName);
            $sProfileName = str_replace('/', '_', $sProfileName);
            $sProfileName = str_replace('.', '-', $sProfileName);

            return $sProfileName;
        }
    }
